@@ -9,7 +9,7 @@
     Released under the GNU Public Licence, Version 2.
 */
 
-#define VERSION "2.4.6"
+#define VERSION "2.4.7"
 
 #include "ksymoops.h"
 #include <ctype.h>
@@ -20,11 +20,12 @@
 #include <string.h>
 #include <unistd.h>
 #include <sys/utsname.h>
+#include <endian.h>
 
 char *prefix;
-char *path_nm = "/usr/bin/nm";          /* env KSYMOOPS_NM */
+char *path_nm = INSTALL_PREFIX"/bin/"CROSS"nm";          /* env KSYMOOPS_NM */
 char *path_find = "/usr/bin/find";      /* env KSYMOOPS_FIND */
-char *path_objdump = "/usr/bin/objdump";    /* env KSYMOOPS_OBJDUMP */
+char *path_objdump = INSTALL_PREFIX"/bin/"CROSS"objdump";    /* env KSYMOOPS_OBJDUMP */
 int debug = 0;
 int errors = 0;
 int warnings = 0;
@@ -56,7 +57,7 @@ regmatch_t *re_unbracketed_address_pmatch;
 
 static void usage(void)
 {
-    static const char usage_text[] = 
+    static const char usage_text[] =
 	   "\t[-v vmlinux] [--vmlinux=vmlinux]\n\t\t\t\t\tWhere to read vmlinux\n"
 	   "\t[-V] [--no-vmlinux]\t\tNo vmlinux is available\n"
 	   "\t[-k ksyms] [--ksyms=ksyms]\tWhere to read ksyms\n"
@@ -144,6 +145,16 @@ static void usage(void)
 	   "-M"
 #endif
 	   "\n"
+#ifdef DEF_TARGET
+	   "\t\t\t"
+	   "-t " DEF_TARGET
+	   "\n"
+#endif
+#ifdef DEF_ARCH
+	   "\t\t\t"
+	   "-a " DEF_ARCH
+	   "\n"
+#endif
 	   "\n"
 	  ;
     printf("Version " VERSION "\n");
@@ -704,6 +715,25 @@ int main(int argc, char **argv)
 #ifdef DEF_MAP
     options.system_map = DEF_MAP;
 #endif
+#ifdef DEF_TARGET
+    options.target = DEF_TARGET;
+    {
+	int target_byteorder;
+
+	if (strstr(options.target, "-big"))
+	    target_byteorder = __BIG_ENDIAN;
+	else if (strstr(options.target, "-little"))
+	    target_byteorder = __LITTLE_ENDIAN;
+	else
+	    target_byteorder = __BYTE_ORDER;
+
+	if (target_byteorder != __BYTE_ORDER)
+	    options.endianess = 1;
+    }
+#endif
+#ifdef DEF_ARCH
+    options.architecture = DEF_ARCH;
+#endif
 
     /* Not all include files define __u16, __u32, __u64 so do it the hard way.
      * Any decent optimizing compiler will discard these tests at compile time
