@@ -1,7 +1,7 @@
 /*
     oops.c.
 
-    Oops processing for ksymoop.
+    Oops processing for ksymoops.
 
     Copyright 1999 Keith Owens <kaos@ocs.com.au>.
     Released under the GNU Public Licence, Version 2.
@@ -638,6 +638,10 @@ static void Oops_set_regs(const char *line, const char *p,
 static regex_t     re_Oops_cris_regs;
 static regmatch_t *re_Oops_cris_regs_pmatch;
 static int         s390_reg_num = -1;
+#define MIPS64_CKSEG0_MASK    	0xffffffff80000000ULL
+#define MIPS64_CKSEG0_BASE    	0xffffffff80000000ULL
+#define MIPS64_KSEG0_BASE    	0x0000000080000000ULL
+#define MIPS64_XKPHYS5_BASE    	0xa800000000000000ULL
 
 /* All address arithmetic is done in 64 bit mode, truncate to the result to
  * a valid range for the target.
@@ -654,6 +658,20 @@ static addr_t Oops_truncate_address(addr_t address, const OPTIONS *options)
 	    u32 &= 0x7fffffff;  /* Really 31 bit addressing on i370/s390 */
 	address = u32;
     }
+    if (options->target &&
+	strstr(options->target, "mips") &&
+	strstr(options->target, "64")) {
+	/*
+	 * Some address gymnastics are necessary due to the weird way
+	 * in which mips64 kernel are built as elf32 then converted
+	 * to elf64 after linking.
+	 */
+	if ((address & MIPS64_CKSEG0_MASK) == MIPS64_CKSEG0_BASE ||
+	    (address & MIPS64_CKSEG0_MASK) == MIPS64_KSEG0_BASE) {
+	    /* normalise physical addrs in 32-bit kernel segments */
+	    address = (address & ~MIPS64_CKSEG0_MASK) | MIPS64_XKPHYS5_BASE;
+	}
+    }
     address &= truncate_mask;
     return(address);
 }
@@ -687,7 +705,7 @@ static const char *Oops_arch_to_eip(const OPTIONS *options)
     if (strstr(options->target, "x86_64") ||
 	strstr(options->target, "x86-64") ||
 	strstr(options->target, "x8664"))
-	    return("RIP"); 
+	    return("RIP");
     return("???");
 }
 
@@ -708,6 +726,8 @@ static char *Oops_eip(const char *line, char ***string, int string_max,
     static regmatch_t *re_Oops_eip_kdb_pmatch;
     static regex_t     re_Oops_eip_ia64;
     static regmatch_t *re_Oops_eip_ia64_pmatch;
+    static regex_t     re_Oops_mca_ip_ia64;
+    static regmatch_t *re_Oops_mca_ip_ia64_pmatch;
     static regex_t     re_Oops_eip_cris;
     static regmatch_t *re_Oops_eip_cris_pmatch;
     static regex_t     re_Oops_eip_other;
@@ -811,7 +831,7 @@ static char *Oops_eip(const char *line, char ***string, int string_max,
 
     i = regexec(&re_Oops_eip_ia64, line, re_Oops_eip_ia64.re_nsub+1,
 	re_Oops_eip_ia64_pmatch, 0);
-    DEBUG(4, "regexec ia64 %d", i);
+    DEBUG(4, "regexec eip_ia64 %d", i);
     if (i == 0) {
 	re_string_check(re_Oops_eip_ia64.re_nsub+1, string_max,
 	    procname);
@@ -820,6 +840,26 @@ static char *Oops_eip(const char *line, char ***string, int string_max,
 	return((*string)[re_Oops_eip_ia64.re_nsub]);
     }
 
+    /* MCA 'IP' line for ia64.  [x]ip at start of line, space, '('regname'),
+     * spaces, :, space, 0x, UNBRACKETED_ADDRESS.
+     */
+    RE_COMPILE(&re_Oops_mca_ip_ia64,
+	    "[xi]ip *\\([^)]*\\) *: *"
+	    "0x" UNBRACKETED_ADDRESS,
+	REG_NEWLINE|REG_EXTENDED|REG_ICASE,
+	&re_Oops_mca_ip_ia64_pmatch);
+
+    i = regexec(&re_Oops_mca_ip_ia64, line, re_Oops_mca_ip_ia64.re_nsub+1,
+	re_Oops_mca_ip_ia64_pmatch, 0);
+    DEBUG(4, "regexec mca_ip_ia64 %d", i);
+    if (i == 0) {
+	re_string_check(re_Oops_mca_ip_ia64.re_nsub+1, string_max,
+	    procname);
+	re_strings(&re_Oops_mca_ip_ia64, line, re_Oops_mca_ip_ia64_pmatch,
+	    string);
+	return((*string)[re_Oops_mca_ip_ia64.re_nsub]);
+    }
+
     /* SGI kdb backtrace EIP.  At start of line, 0x unbracketed
      * address, 0x unbracketed address, anything, '+0x'.  The second
      * unbracketed address is the EIP.
@@ -876,10 +916,10 @@ static char *Oops_eip(const char *line, char ***string, int string_max,
     /* Oops 'EIP:' line for other architectures */
     RE_COMPILE(&re_Oops_eip_other,
 		"^("
-    		  "EIP: +.*"	/* i386 */
-    		  "|RIP: +.*"	/* x86_64 */
-    		  "|PC *= *"	/* m68k, alpha */
-    		  "|pc *: *"	/* arm */
+		  "EIP: +.*"	/* i386 */
+		  "|RIP: +.*"	/* x86_64 */
+		  "|PC *= *"	/* m68k, alpha */
+		  "|pc *: *"	/* arm */
 		")"
 		BRACKETED_ADDRESS
 		,
@@ -1153,10 +1193,10 @@ static void Oops_set_ia64_b0(const char *line, char ***string, int string_max, S
     static const char procname[] = "Oops_ia64_b0";
 
     /* Oops 'b0' line for ia64, b0 : unbracketed address, may be
-     * repeated for b6, b7.
+     * repeated for b6, b7.  MCA has b0 '('regname')' : 0x unbracketed address.
      */
     RE_COMPILE(&re_Oops_ia64_b0,
-	    "^b[0-7] *: *"
+	    "^b[0-7] *(\\([^)]*\\) *)?: *(0x)?"
 	    UNBRACKETED_ADDRESS,
 	REG_NEWLINE|REG_EXTENDED|REG_ICASE,
 	&re_Oops_ia64_b0_pmatch);
@@ -1449,7 +1489,8 @@ static const char *Oops_trace(const char *line)
 }
 
 /* Process a trace call line, extract addresses */
-static void Oops_trace_line(const char *line, const char *p, SYMBOL_SET *ss)
+static void Oops_trace_line(const char *line, const char *p, SYMBOL_SET *ss,
+			    const OPTIONS *options)
 {
     char **string = NULL;
     regex_t *pregex;
@@ -1482,8 +1523,11 @@ static void Oops_trace_line(const char *line, const char *p, SYMBOL_SET *ss)
     /* Loop over (un|rev)?bracketed addresses */
     while (1) {
 	if (regexec(pregex, p, pregex->re_nsub+1, pregmatch, 0) == 0) {
+	    addr_t addr;
 	    re_strings(pregex, p, pregmatch, &string);
-	    add_symbol(ss, string[1], 'T', 1, "Trace;");
+	    addr = hexstring(string[1]);
+	    addr = Oops_truncate_address(addr, options);
+	    add_symbol_n(ss, addr, 'T', 1, "Trace;");
 	    p += pregmatch[0].rm_eo;
 	}
 	else if (strncmp(p, "from ", 5) == 0)
@@ -1720,6 +1764,7 @@ static int Oops_print(const char *line, const char **text, char ***string,
 	      "[^ ]+ kernel: +"		/* syslogd */
 	    "|<[0-9]+>"			/* kmsg */
 	    "|[0-9]+\\|[^|]+\\|[^|]+\\|[^|]+\\|"	/* syslog-ng */
+	    "|[0-9][AC] [0-9]{3} [0-9]{3}[cir][0-9]{2}:"	/* SNIA */
 	    ")",
 	REG_NEWLINE|REG_EXTENDED|REG_ICASE,
 	&re_Oops_prefix_pmatch);
@@ -1931,12 +1976,33 @@ static int Oops_print(const char *line, const char **text, char ***string,
     /* SGI kdb */ "|ds *="
     /* any other SGI kdb lines to print? */
 
-    /* ia64 */	    "|psr *:"
-    /* ia64 */	    "|unat *: "
-    /* ia64 */	    "|rnat *: "
-    /* ia64 */	    "|ldrs *: "
-    /* ia64 */	    "|[bfr][0-9]+ *: "
+    /* ia64 */	    "|psr "
+    /* ia64 */	    "|unat  "
+    /* ia64 */	    "|rnat "
+    /* ia64 */	    "|ldrs "
+    /* ia64 */	    "|xip "
+    /* ia64 */	    "|iip "
+    /* ia64 */	    "|ipsr "
+    /* ia64 */	    "|ifa "
+    /* ia64 */	    "|pr "
+    /* ia64 */	    "|itc "
+    /* ia64 */	    "|ifs "
+    /* ia64 */	    "|bsp "
+    /* ia64 */	    "|[bfr][0-9]+ "
+    /* ia64 */	    "|irr[0-9] "
     /* ia64 */	    "|General Exception"
+    /* ia64 */	    "|MCA"
+    /* ia64 */	    "|SAL"
+    /* ia64 */	    "|Processor State"
+    /* ia64 */	    "|Bank [0-9]+ "
+    /* ia64 */	    "|Register Stack"
+    /* ia64 */	    "|Processor Error Info"
+    /* ia64 */	    "|proc err map"
+    /* ia64 */	    "|proc state param"
+    /* ia64 */	    "|proc lid"
+    /* ia64 */	    "|processor structure"
+    /* ia64 */	    "|check info"
+    /* ia64 */	    "|target identifier"
     /* any other ia64 lines to print? */
 
     /* CRIS */	    "|IRP *: "
@@ -2255,7 +2321,7 @@ static FILE *Oops_next_file(OPTIONS *options)
 	if (f)
 	    fclose_local(f, procname);
 	f = NULL;
-	if (strcmp(*(options->filename), "/dev/null") == 0 || 
+	if (strcmp(*(options->filename), "/dev/null") == 0 ||
 	    regular_file(*(options->filename), procname))
 	    f = fopen_local(*(options->filename), "r", procname);
 	if (f)
@@ -2340,7 +2406,7 @@ int Oops_read(OPTIONS *options)
 		     * the equivalent of a trace entry.
 		     */
 		    Oops_set_eip(string[1], &eip, &ss_format, me, ibfd, options);
-		    Oops_trace_line(line, string[2], &ss_format);
+		    Oops_trace_line(line, string[2], &ss_format, options);
 		}
 		if (Oops_eip_sparc_spinlock(text, &string, MAX_STRINGS,
 					    options)) {
@@ -2350,7 +2416,7 @@ int Oops_read(OPTIONS *options)
 		     */
 		    Oops_set_eip(string[2], &eip, &ss_format, me, ibfd, options);
 		    Oops_set_eip(string[3], &eip, &ss_format, me, ibfd, options);
-		    Oops_trace_line(line, string[1], &ss_format);
+		    Oops_trace_line(line, string[1], &ss_format, options);
 		}
 		if ((start = Oops_mips_ra(text, &string, MAX_STRINGS)))
 		    Oops_set_mips_ra(start, &ss_format);
@@ -2361,7 +2427,7 @@ int Oops_read(OPTIONS *options)
 		if ((start = Oops_s390_psw(text, &string, MAX_STRINGS)))
 		    Oops_set_s390_psw(start, &ss_format, me, ibfd, options);
 		if ((start = Oops_trace(text)))
-		    Oops_trace_line(text, start, &ss_format);
+		    Oops_trace_line(text, start, &ss_format, options);
 		if ((start = Oops_regs(text)))
 		    Oops_set_regs(text, start, &ss_format, options);
 		if ((start = Oops_s390_regs(text)))
